require 'spec_helper'
require 'object_spy/rack/middleware'

describe ObjectSpy::Rack::Middleware do
  let(:app) { double('app') }

  describe '#initialize' do

    it 'accepts and assigns @app' do
      described_class.new(app).app.should eq(app)
    end
  end

  describe '#call' do
    let(:middleware)  { described_class.new(app) }
    let(:env)         { double('env', :[] => double('access').as_null_object) }

    it 'invokes #call on @app' do
      expect(app).to receive(:call).with(env)

      middleware.call(env)
    end

    it 'invokes ObjectSpy.spy' do
      expect(app).to receive(:call).with(env)

      expect(ObjectSpy).to receive(:spy)

      middleware.call(env)
    end

    it 'rescues from errors' do
      expect(app).to receive(:call).with(env)

      expect(ObjectSpy).to receive(:spy).and_raise(RuntimeError)

      expect { middleware.call(env) }.not_to raise_error
    end
  end

end
